# -*- coding: utf-8 -*-
##############################################################################
#
#    account module for OpenERP
#    Copyright (C) 2010 Tecnoba S.L. (http://www.tecnoba.com)
#       Pere Ramon Erro Mas <pereerro@tecnoba.com> All Rights Reserved.
#
#
#    account_contact OpenERP module is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    account_contact OpenERP module is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################


from osv import osv, fields
from datetime import datetime, date, timedelta
from tools.translate import _
import subprocess
import tempfile
import base64
import sys,os

class document_type(osv.osv):
    _name = 'document.type'
    
    _columns = {
        'name' : fields.char('Name', size=200,),
        'always_signed' : fields.boolean('Always signed',),
    }
    
document_type()

DOC_STATES = [('empty','Empty'),        # no file
              ('checked','Checked'),    # the document is ok
              ('unsigned','Unsigned'),  # the sign is mandatory
              ('signed','Signed'),      # a check is needed
              ('expired','Expired'),    # the valid_to date is in the past
              ]

class document_file(osv.osv):
    _inherit = 'ir.attachment'
    
    def get_signature(self, cr, uid, ids, field_name, arg, context=None):
        ret = dict.fromkeys(ids, False);
        for item in self.browse(cr, uid, ids):
            try:
                if item.signer and item.datas:
                    f = tempfile.NamedTemporaryFile(delete=False)
                    f.write(base64.decodestring(item.datas))
                    f.close()
                    ret[item.id] = subprocess.check_output(["checkPDFSignedByNIF.jar", f.name, item.signer,])
                    os.unlink(f.name)                    
            except:
                print "Unexpected error:", sys.exc_info()[0]
        return ret
    
    def refresh_signature(self, cr, uid, ids, context=None):
        dicci = self.get_signature(cr, uid, ids, None, None, context=context)
        for lid,sign in dicci.items():
            cr.execute("UPDATE ir_attachment SET sign=%s WHERE id=%s", (sign,lid))
        return True
    
    def _get_document_state(self, cr, uid, ids, field_name, arg, context=None):
        ret = {}
        for item in self.browse(cr, uid, ids, context=context):
            ret[item.id] = 'unsigned'
            if not item.datas or not item.datas_fname:
                ret[item.id] = 'empty'
            elif item.valid_to and item.valid_to < datetime.todays().strftime('%Y-%m-%d'):
                ret[item.id] = 'expired'
            else:
                if item.sign:
                    ret[item.id] = 'signed'
                sign_needed = item.document_type_id and item.document_type_id.always_signed
                if item.checked and (ret[item.id] == 'signed' or not sign_needed):
                    ret[item.id] = 'checked'
        return ret
    
    _columns = {
        'contact_ids' : fields.many2many('res.partner.contact', 'contact_document_doc_contact_rel', 'doc_id', 'contact_id', string='Contacts',),
        'document_type_id' : fields.many2one('document.type', string='Document type',),
        'valid_to' : fields.date('Valid to',),
        'sign' : fields.function(
                get_signature,
                type='text',method=True,
                string='Signature',
                store={
                       'ir.attachment' : (lambda self,cr,uid,ids,ctx={}: ids, ["datas","store_fname","signer","db_datas"], 10),
                    }),
        'checked' : fields.boolean('Checked', help="True if the file has been checked as correct",),
        'note_to_partner' : fields.text('Note', help="Note addressed to the partner",),
        'signer' : fields.char('Serial Number', size=15, help="Signer serial number",),
        'doc_state' : fields.function(_get_document_state, method=True, type='selection', selection=DOC_STATES, string='Document State',),
    }
    
    def write(self, cr, uid, ids, vals, context=None):
        if 'datas' in vals and 'checked' not in vals:
            vals['checked'] = False # The file has changed
        return super(document_file, self).write(cr, uid, ids, vals, context=context)
    
document_file()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

