# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import time

from osv import fields, osv
from tools.translate import _
import zipfile
from StringIO import StringIO
import base64
import netsvc
from locale import * 

_RECIBO_CSB = 'RECIBO_CSB'

class membership_cancel_repeated_invoices(osv.osv_memory):
    _name = 'membership.cancel_repeated_invoices'
    
    _columns = {
        'product_id': fields.many2one('product.product', string="Membership Product", required=True,),
        'date': fields.date('Operation date', help='This date will be used as the invoice date for Refund Invoice and Period will be chosen accordingly!'),
        'period': fields.many2one('account.period', 'Force period'),
        'journal_id': fields.many2one('account.journal', 'Refund Journal', help='You can select here the journal to use for the refund invoice that will be created. If you leave that field empty, it will use the same journal as the current invoice.'),
        'description': fields.char('Description', size=128, required=True),
    }

    def _get_journal(self, cr, uid, context=None):
        obj_journal = self.pool.get('account.journal')
        if context is None:
            context = {}
        journal = obj_journal.search(cr, uid, [('type', '=', 'sale_refund')])
        if context.get('type', False):
            if context['type'] in ('in_invoice', 'in_refund'):
                journal = obj_journal.search(cr, uid, [('type', '=', 'purchase_refund')])
        return journal and journal[0] or False

    _defaults = {
        'date': lambda *a: time.strftime('%Y-%m-%d'),
        'journal_id': _get_journal,
        'filter_refund': 'modify',
    }
    
    def cancel_repeated(self, cr, uid, ids, context = None):
        for wizard in self.browse(cr, uid, ids):
            partner_id = True
            while partner_id:
                query = """
                    SELECT ai.partner_id,ai.id,ai.state
                    FROM account_invoice_line AS ail
                        INNER JOIN account_invoice AS ai ON
                            ail.invoice_id = ai.id AND
                            ail.product_id = %(product_id)s AND
                            ai.state in ('draft','proforma','proforma2','open')
                        LEFT JOIN account_invoice AS ai_2 ON
                            ai.partner_id = ai_2.partner_id AND
                            ai.id <> ai_2.id
                        LEFT JOIN account_invoice_line AS ail_2 ON
                            ail_2.invoice_id = ai_2.id AND
                            ail_2.product_id = ail.product_id
                    WHERE ail_2.id IS NOT NULL
                    LIMIT 1
                """
                cr.execute(query, {
                        'product_id': wizard.product_id.id,})
                partner_id = 0
                for (partner_id,invoice_id,istate) in cr.fetchall():
                    if istate in ('draft','proforma','proforma2'):
                        self.pool.get('account.invoice').unlink(cr, uid, [invoice_id], context = context)
                    else:
                        wizard_id = self.pool.get('account.invoice.refund').create(cr, uid, {
                                'date': wizard.date,
                                'period': wizard.period.id,
                                'journal_id': wizard.journal_id.id,
                                'description': wizard.description,
                                'filter_refund': 'cancel', }, context = context)
                        self.pool.get('account.invoice.refund').invoice_refund(cr, uid, [wizard_id], context = {'active_ids': [invoice_id,]},)
        return True
        
    
membership_cancel_repeated_invoices()

class membership_q19_import(osv.osv_memory):
    _name = 'membership.q19_import'
    
    _columns = {
        'product_id' : fields.many2one('product.product', string="Membership Product", required=True,),
        'q19_file' : fields.binary('Q19 File', required=True,),
        'communication2' : fields.char('Communication2', size=100,),
        'currency_id' : fields.many2one('res.currency','Currency',required=True,),
        'payment_mode_id' : fields.many2one('payment.mode', string='Payment Mode', required=True,),        
        'account_client' : fields.many2one('account.account', string='Client account', required=True,),
        'account_bank' : fields.many2one('account.account', string='Bank account', required=True,),
        'period_id': fields.many2one('account.period', 'Period', required=True, select=2),
        'journal_id': fields.many2one('account.journal', 'Journal', required=True, select=1),
        'date' : fields.date('Date', required=True,),
        'bank_state' : fields.many2one('res.partner.bank.type',string='Bank type',help="Bank type to create",),
    }

    def get_lines(self, file):
        ret = []
        vals={}
        val =base64.decodestring(file)
        fp = StringIO()
        fp.write(val)
        fdata = zipfile.ZipFile(fp, 'r')
        for fname in fdata.namelist():
            fr = fdata.open(fname)
            for line in fr:
                code = line and len(line)>=4 and line[0:4] or ''
                if code=='5680':
                    vals = {
                        'nif_ordenant': line[4:13],
                        'id_contact' : line[13:28],
                        'name_contact' : line[28:68],
                        'cc_partner' : line[68:88],
                        'concepte' : line[114:149],
                        'amount' : is_number(line[149:-2]) and atof(line[149:-2]) or 0,
                    }
                def is_number(s):
                    try:
                        atof(s)
                        return True
                    except ValueError:
                        return False                    
                if code=='5686':
                    vals.update({
                        'id_partner' : line[13:28],
                        'name_partner' : line[28:68],
                        'address_1' : line[68:108],
                        'address_2' : line[108:142],
                        'address_3' : line[142:162],
                    })
                    ret.append(vals)
        return ret
                
        
    def import_membership(self, cr, uid, ids, context = None):
        rp_obj = self.pool.get('res.partner')
        bank_obj = self.pool.get('res.bank')
        payment_type_id = self.pool.get('payment.type').search(cr, uid, [('code','=',_RECIBO_CSB)])
        if not payment_type_id:
            raise osv.except_osv(_('Invalid action!'), _("The RECIBO_CSB type payment doesn't exist!"))
        payment_type_id = payment_type_id[0]   
        payment_order_ids = []         
        for wizard in self.browse(cr, uid, ids, context = context):

            payment_order_id = self.pool.get('payment.order').create(cr, uid, {'mode': wizard.payment_mode_id.id,'type': 'receivable'})
            payment_order_ids.append(payment_order_id)
            payment = self.pool.get('payment.order').browse(cr, uid, payment_order_id,
                    context=context)
            
            lines = self.get_lines(wizard.q19_file)

            
            for line in lines:
                
                bank_code = line['cc_partner'][0:4]
                bank_ids = bank_obj.search(cr, uid, [('code','=',bank_code)])
                if not bank_ids:
                    bank_id = bank_obj.create(cr, uid, {'name': bank_code, 'code': bank_code,}, context = context)
                else:
                    bank_id = bank_ids[0]

                partner_ids = rp_obj.search(cr, uid, [('name','=',line['name_partner']),])
                if not partner_ids:
                    partner_id = rp_obj.create(cr, uid, {
                        'name': line['name_partner'],
                    })
                    partner_ids = [partner_id]
                else:
                    partner_id = partner_ids[0]
                    
                
                address_ids = self.pool.get('res.partner.address').search(cr, uid, [
                                ('partner_id','=',partner_id),
                                ('type','=','invoice'),
                                ('street','=',line['address_1']),
                                ('city','=',line['address_2']),
                                ('zip','=',line['address_3']), ])
                if not address_ids:
                    address_id = self.pool.get('res.partner.address').create(cr, uid, {
                            'partner_id': partner_id,
                            'type': 'invoice',
                            'street': line['address_1'] or '----',
                            'city': line['address_2'],
                            'zip': line['address_3'],
                        })
                else:
                    address_id = address_ids[0]
                self.pool.get('res.partner.contact').create(cr, uid, {'name': line['name_contact'], 'job_ids': [(0,0,{
                                'address_id': address_id, 'function': 'Tutorized',})]})
                
                bank_cc_ids = self.pool.get('res.partner.bank').search(cr, uid, [('acc_number','=',line['cc_partner'])])
                if not bank_cc_ids:
                    bank_cc_id = self.pool.get('res.partner.bank').create(cr, uid, {
                        'partner_id': partner_id, 
                        'name': 'from q19 import wizard', 
                        'acc_number': line['cc_partner'], 
                        'bank': bank_id,
                        'state': wizard.bank_state.code, })
                else:
                    bank_cc_id = bank_cc_ids[0]
                    
                if not self.pool.get('account.invoice.line').search(cr, uid, [('product_id','=',wizard.product_id.id),('invoice_id.state','=','draft')]):
                    invoice_id = rp_obj.create_membership_invoice(cr, uid, partner_ids, product_id=wizard.product_id.id, datas={'amount': line['amount'], }, context=context)
                
                #===============================================================
                # mv_id = self.pool.get('account.move').create(cr, uid, {
                #    'period_id' : wizard.period_id.id,
                #    'journal_id' : wizard.journal_id.id,
                #    'date' : wizard.date,
                #    'line_id': [
                #        (0,0,{'partner_id': partner_id, 'name': line['concepte'], 'debit': line['amount'], 'credit': 0, 'account_id': wizard.account_bank.id, 'period_id': wizard.period_id.id, 'journal_id': wizard.journal_id.id, 'date': wizard.date, }),
                #        (0,0,{'partner_id': partner_id, 'name': line['concepte'], 'debit': 0, 'credit': line['amount'], 'account_id': wizard.account_client.id, 'period_id': wizard.period_id.id, 'journal_id': wizard.journal_id.id, 'date': wizard.date, }),
                #        ]
                #    })      
                # move = self.pool.get('account.move').browse(cr, uid, mv_id)
                #===============================================================
                pl_id = self.pool.get('payment.line').create(cr,uid,{
                    'amount_currency': line['amount'],
                    'order_id': payment.id,
                    'partner_id': partner_id,
                    'communication': line['concepte'],
                    'communication2': wizard.communication2,
                    'currency': wizard.currency_id.id,
#                    'move_line_id' : [x.id for x in move.line_id if x.credit>0][0], 
                    'bank_id' : bank_cc_id,
                    }, context=context)
                
                
            #===================================================================
            # self.pool.get('account.invoice').write(cr, uid, invoice_ids, {'payment_type': payment_type_id,})
            # for i in invoice_ids:
            #    wf_service = netsvc.LocalService("workflow")
            #    wf_service.trg_validate(uid, 'account.invoice', i, 'invoice_open', cr)
            #===================================================================
                                      
        tree_view_id = self.pool.get('ir.ui.view').search(cr, uid, [
            ('model','=','payment.order'),
            ('type','=','tree'),], limit=1,)[0]
        form_view_id = self.pool.get('ir.ui.view').search(cr, uid, [
            ('model','=','payment.order'),
            ('type','=','form'),], limit=1,)[0]
        
        return {
            'name' : 'Order payment created',
            'type' : 'ir.actions.act_window',
            'view_type' : 'form',
            'view_mode' : 'tree,form',
            'view_id': False,
            'res_model' : 'payment.order',
            'domain' : [('id','in',payment_order_ids)],
            'view_ids' : [('tree',tree_view_id),('form',form_view_id)],
        }
    
membership_q19_import()


# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4: