# -*- coding: utf-8 -*-
##############################################################################
#
#    Pere Ramon Erro Mas
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import fields, osv
from tools.translate import _
import tempfile

STATE = [
    ('none', 'Non Member'),
    ('canceled', 'Cancelled Member'),
    ('suspend', 'Suspended Member'),
    ('old', 'Old Member'),
    ('waiting', 'Waiting Member'),
    ('invoiced', 'Invoiced Member'),
    ('free', 'Free Member'),
    ('paid', 'Paid Member'),
]

STATES_ON = ('paid','free')

def username(partner,joomla_site):
    return (not joomla_site.country_code) and partner.vat\
        and partner.vat[2:] or partner.vat

def change_joomla_group(mysql_cr, table_prefix, uname, old_group_title, new_group_title):
    # add user to new group
    query = """
    INSERT IGNORE INTO %(prefix)suser_usergroup_map (user_id,group_id)
        SELECT u.id AS uid,g.id AS gid FROM %(prefix)susers AS u,%(prefix)susergroups as g
        WHERE u.username=%%(uname)s AND g.title=%%(gtitle)s
    """ % {'prefix': table_prefix,}
    mysql_cr.execute(query, {'uname': uname, 'gtitle': new_group_title,})
    # remove user from old group
    query = """
    DELETE FROM ug
    USING %(prefix)suser_usergroup_map AS ug INNER JOIN
        %(prefix)susers AS u ON ug.user_id=u.id AND u.username=%%(uname)s INNER JOIN
        %(prefix)susergroups AS g ON ug.group_id=g.id AND g.title=%%(gtitle_old)s 
    """ % {'prefix': table_prefix,}
    mysql_cr.execute(query, {'uname': uname, 'gtitle_old': old_group_title,})
    
                                  
class mysql_connection(osv.osv):
    _name = 'mysql.connection'
                
    def name_get(self, cr, uid, ids, context=None):
        ret = []
        for item in self.browse(cr, uid, ids, context=context):
            ret.append( (item.id, "%s@%s:%s/%s" % (item.mysql_user, item.mysql_host, item.mysql_port, item.mysql_dbname) ) )
        return ret
    
    _columns = {
        'mysql_host': fields.char('Mysql host', size=200, required=True,),
        'mysql_port': fields.integer('Mysql port', required=True,),
        'mysql_dbname': fields.char('Mysql Database name', size=100, required=True,),
        'mysql_user': fields.char('Mysql user', size=50,),
        'mysql_password': fields.char('Mysql password', size=50,),
        'last_status' : fields.char('Last Status', size=100,),
        'use_ssl' : fields.boolean('Use SSL',),
        'mysql_client_cert' : fields.many2one('crypto.certificate', string='Mysql Client Certificate',
                                              help='Certificate to use in connections with Mysql Server',),
        'mysql_client_privkey' : fields.many2one('crypto.pairkey', string='Mysql Client Private Key',),
        'mysql_ca_cert' : fields.many2one('crypto.certificate', string='Mysql CA Certificate',
                                              help='Certificate common with the server to verify  use in connections with Mysql Server',),
    }
    
    def get_connection(self, cr, uid, conn_id, context=None):
        conn = self.browse(cr, uid, conn_id, context=context)
        tdb = False
        try:
            import MySQLdb
        except Exception, e:
            self.write(cr, uid, conn_id, {'last_status': _('MySQLdb Package Not Installed.'),}),
        if conn.use_ssl:
            ssl_cert = tempfile.NamedTemporaryFile(bufsize=0,delete=False)
            ssl_cert.write(conn.mysql_client_cert.crt)
            ssl_key = tempfile.NamedTemporaryFile(bufsize=0,delete=False)
            ssl_key.write(conn.mysql_client_privkey.key)    # Warning!!!! Tested on Ubuntu Linux Precise the file created by
                                                            # NamedTemporaryFile is only readable for the user who execute the server
                                                            # Not Tested on others platforms
            ssl_ca = tempfile.NamedTemporaryFile(bufsize=0,delete=False)
            ssl_ca.write(conn.mysql_ca_cert.crt)
            ssl_options = {'ca': ssl_ca.name, 'cert': ssl_cert.name, 'key': ssl_key.name,}
            print "ssl_options: ", ssl_options
        else:
            ssl_options = None
        try:
            tdb = MySQLdb.connect(host = conn.mysql_host,port = conn.mysql_port, db = conn.mysql_dbname,
                                  user = conn.mysql_user, passwd = conn.mysql_password or '', ssl=ssl_options)
        except Exception, e:
            self.write(cr, uid, conn_id, {'last_status': _("Can't connect with MySQL. Please, revise configuration or if MySQL server is up."),})
        finally:
            if conn.use_ssl:
                ssl_cert.close()
                ssl_key.close()
                ssl_ca.close()            
        if not tdb:
            raise osv.except_osv(_("Error"),_("Couldn't connect to MySQL. Contact with the administrator: %s" % e))
        self.write(cr, uid, conn_id, {'last_status': _("Connection successfully."),})
        return tdb
            
mysql_connection()    

class joomla_group(osv.osv):
    _name = 'membership.joomla_group'
    
    _columns = {
        'name' : fields.char('Name', size=200,),
        'joomla_site_id' : fields.many2one('membership.joomla_site', ondelete='cascade',),
    }
joomla_group()

class joomla_site(osv.osv):
    _name = 'membership.joomla_site'
    _inherit = 'mysql.connection'
    
    _columns = {
        'company_id': fields.many2one('res.company', string='Company', ondelete='set null',),
        'table_prefix' : fields.char('Table prefix', size=30,),
        'joomla_group' : fields.many2one('membership.joomla_group', string='Joomla group',
                                         help='Joomla group exclusive to memberships', domain="[('joomla_site_id','=',active_id)]",),
        'joomla_group_old' : fields.many2one('membership.joomla_group', string='Old members Joomla group',
                                             help='Name of the Joomla group exclusive to old memberships',
                                             domain="[('joomla_site_id','=',active_id)]",),
        'country_code' : fields.boolean('Country code', help='Add VAT country code to Joomla username'),
    }
    
    def get_company_sites(self, cr, uid, context=None):
        user_obj = self.pool.get('res.users')
        ids = self.search(cr, uid, [('company_id','in',
            [x[0] for x in user_obj.get_current_company(cr, uid) ])])
        return ids
    
    def check_groups(self, cr, uid, ids, context=None):
        jg_obj = self.pool.get('membership.joomla_group')
        for js in self.browse(cr, uid, ids, context=context):
            mysql_conn = self.get_connection(cr, uid, js.id)
            with mysql_conn:
                mycr = mysql_conn.cursor()
                # creating temporary table of users from openerp memberships
                mycr.execute("SELECT title FROM %susergroups" % (js.table_prefix,))
                titles_out = [title for (title,) in mycr.fetchall()]
                joomla_group_ids_to_remove = []
                titles_in = [] # titles (Joomla group names) in openerp
                for jg in jg_obj.browse(cr, uid, jg_obj.search(cr, uid, [('joomla_site_id','=',js.id)])):
                    if jg.name not in titles_out:
                        joomla_group_ids_to_remove.append(jg.id)
                    titles_in.append(jg.name)
                # remove groups not in Joomla
                jg_obj.unlink(cr,uid, joomla_group_ids_to_remove)
                # create groups in Joomla but not in openerp
                for title_out in titles_out:
                    if title_out not in titles_in:
                        jg_obj.create(cr, uid, {'name': title_out, 'joomla_site_id': js.id})
        return True
                
    def check_users(self, cr, uid, ids, context=None):
        """This Function synchronize joomla users with Memberships.
        @param self: The object pointer
        @param cr: the current row, from the database cursor,
        @param uid: the current user’s ID for security checks,
        @param ids: List of Joomla Sites IDs
        @param context: A standard dictionary for contextual values
        @param return: Nothing
        """
        js_obj = self.pool.get('membership.joomla_site')        
        rp_obj = self.pool.get('res.partner')
        for js in self.browse(cr, uid, ids, context=context):
            mysql_conn = self.get_connection(cr, uid, js.id)
            with mysql_conn:
                mycr = mysql_conn.cursor()
                js_obj.check_groups_exist(cr, uid, js, mycr, context=context)
                # creating temporary table of users from openerp memberships
                mycr.execute("CREATE TEMPORARY TABLE unames (uname char(100) PRIMARY KEY,name char(200), email char(200))")
                partners = [x for x in rp_obj.browse(cr, uid,
                        rp_obj.search(cr, uid, [('membership_state','in',STATES_ON),('vat','!=',False)])
                            , context=context) if username(x,js)]
                if partners:
                    # populate temporary table
                    mycr.execute("INSERT IGNORE INTO unames VALUES "+','.join(['(%s,%s,%s)'] * len(partners)),
                                 sum([[username(x,js),x.name,x.email] for x in partners],[]) )
                # no email if it is into 
                mycr.execute("UPDATE unames INNER JOIN %(prefix)susers AS u ON unames.email=u.email SET unames.email=''" % {'prefix': js.table_prefix,})
                # create new users
                mycr.execute("""INSERT IGNORE INTO %(prefix)susers (username,name,email,params)
                    SELECT unames.uname,unames.name,unames.email,'' FROM unames
                        LEFT JOIN %(prefix)susers AS u ON unames.uname=u.username
                    WHERE u.id IS NULL""" % {'prefix': js.table_prefix,})
                
                # change group all users in Joomla membership group who aren't memberships in openerp to a group with less perms
                query = """
                UPDATE IGNORE %(prefix)suser_usergroup_map AS ug INNER JOIN
                    %(prefix)susergroups AS g ON ug.group_id=g.id INNER JOIN
                    %(prefix)susers AS u ON ug.user_id=u.id LEFT JOIN
                    unames ON unames.uname=u.username
                SET ug.group_id=(  SELECT id FROM %(prefix)susergroups WHERE title=%%(gtitle_old)s  )
                WHERE unames.uname IS NULL AND g.title=%%(gtitle)s
                """ % {'prefix': js.table_prefix,}
                mycr.execute(query, {'gtitle': js.joomla_group.name, 
                                     'gtitle_old': js.joomla_group_old.name,})
                
                # delete users in Joomla membership group who aren't memberships in openerp
                query = """
                DELETE FROM ug
                USING %(prefix)suser_usergroup_map AS ug INNER JOIN
                    %(prefix)susers AS u ON ug.user_id=u.id INNER JOIN
                    %(prefix)susergroups AS g ON ug.group_id=g.id AND g.title=%%(gtitle)s LEFT JOIN 
                    unames ON u.username=unames.uname
                WHERE unames.uname IS NULL
                """ % {'prefix': js.table_prefix,}
                mycr.execute(query, {'gtitle': js.joomla_group.name,})
                
                # insert users in Joomla membership group who really are memberships in openerp
                query = """
                INSERT IGNORE INTO %(prefix)suser_usergroup_map (user_id,group_id)
                    SELECT u.id,g.id
                    FROM %(prefix)susers AS u,%(prefix)susergroups AS g,unames
                    WHERE u.username=unames.uname AND g.title=%%(gtitle)s
                """ % {'prefix': js.table_prefix,}
                mycr.execute(query, {'gtitle': js.joomla_group.name,})            

    def check_groups_exist(self, cr, uid, js, mycr, context=None):
        if not (js.joomla_group and js.joomla_group_old):
            self.write(cr, uid, [js.id], {'last_status': "The Joomla groups aren't defined",})
            raise osv.except_osv(_("Pay attention"),_("The Joomla groups aren't defined"))
        query = "SELECT title FROM %(prefix)susergroups WHERE title=%%(title)s OR title=%%(title_old)s" % {'prefix': js.table_prefix,}
        mycr.execute(query, {'title': js.joomla_group.name, 'title_old': js.joomla_group_old.name,})
        titles = set([title for (title,) in mycr.fetchall()])
        if len(titles) != 2:
            self.write(cr, uid, [js.id], {'last_status': _("The Joomla groups doesn't exist in Mysql Server"),})
            raise osv.except_osv(_("Pay attention"),_("The Joomla groups doesn't exist in Mysql Server"))

joomla_site()

class res_company(osv.osv):
    _name = 'res.company'
    _inherit = 'res.company'
    
    _columns = {
        'joomla_site_ids' : fields.one2many('membership.joomla_site', 'company_id', string='Joomla Sites',),
    }
    
res_company()    

class res_partner(osv.osv):
    _name = 'res.partner'
    _inherit = 'res.partner'

    """
    Lets go to intercept the method which inform about the membership state when there is a change triggered
    """    
    def _membership_state(self, cr, uid, ids, name, args, context=None):
        js_obj = self.pool.get('membership.joomla_site')
        
        f = 'membership_state'
        datas_before = dict((x['id'],x[f]) for x in self.read(cr, uid, ids, [f], context=context))
        ret = super(res_partner, self)._membership_state(cr, uid, ids, name, args, context=context)
        prt_on, prt_off = [],[]
        for prt_id,state_after in ret.items():
            state_before = datas_before[prt_id]
            if state_before not in STATES_ON and state_after in STATES_ON:
                # a new membership
                prt_on.append(prt_id)
            elif state_before in STATES_ON and state_after not in STATES_ON:
                # this partner isn't membership yet
                prt_off.append(prt_id)
        if prt_on or prt_off:
            # there are changes to pass
            company_js_ids = js_obj.get_company_sites(cr, uid, context=context) 
            for js in js_obj.browse(cr, uid, company_js_ids, context=context):
                mysql_conn = js_obj.get_connection(cr, uid, js.id)
                with mysql_conn:
                    mycr = mysql_conn.cursor()
                    js_obj.check_groups_exist(cr, uid, js, mycr, context=context)
                    for partner in self.browse(cr, uid, prt_on+prt_off, context=context):
                        if username(partner,js):
                            if partner.id in prt_on:
                                email = partner.email
                                query = "SELECT username FROM %susers WHERE username=%%s OR email=%%s" % js.table_prefix
                                mycr.execute(query, (username(partner,js), email))
                                uname_exists = False
                                for (uname,) in mycr.fetchall():
                                    uname_exists |= (uname==username(partner,js))
                                    # There are other partner with other username but with the same email
                                    email = ''
                                if not uname_exists:
                                    query = """
                                    INSERT IGNORE INTO %susers (name, username, email)
                                    VALUES (%%s,%%s,%%s) 
                                    """ % js.table_prefix
                                    mycr.execute(query, (partner.name, username(partner,js), partner.email))
                                change_joomla_group(mycr, js.table_prefix, username(partner,js),js.joomla_group_old.name, js.joomla_group.name)
                            elif partner.id in prt_off:
                                change_joomla_group(mycr, js.table_prefix, username(partner,js),js.joomla_group.name, js.joomla_group_old.name)
        return ret
res_partner()
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4: